# frozen_string_literal: true

require 'dry-validation'

module Swamp
  # Operations
  # @example
  #   class MyOperation
  #     include Swamp::Operation
  #
  #     def _call
  #       input[:id]
  #     end
  #
  #     input do
  #       required(:id) { int? }
  #     end
  #   end
  module Operation
    class ValidationError < StandardError; end

    def self.included(base)
      base.extend ClassMethods
    end

    attr_reader :input

    def call(**kwargs)
      before_call!(**kwargs)
      _call
    end

    def before_call!(**kwargs)
      @input = Hash[]
      return unless self.class.input_processor

      self.class.input_processor.call(kwargs).tap do |result|
        raise ValidationError, result.messages unless result.success?

        @input = result.output
      end
    end

    # Extended module
    module ClassMethods
      attr_reader :input_processor

      def call(**kwargs)
        new.call(**kwargs)
      end

      def input(&block)
        @input_processor = Dry::Validation.Schema do
          configure do
            config.input_processor = :sanitizer
          end

          instance_exec(&block)
        end
      end
    end
  end
end

require 'swamp/operation/result'
