# frozen_string_literal: true

require 'dry/core/class_builder'

module Swamp::Operation
  # Operations
  # @example
  #   class MyOperation
  #     include Swamp::Operation
  #     include Swamp::Operation::Result[:id]
  #
  #     def _call
  #       result.id = input[:id]
  #     end
  #
  #     input do
  #       required(:id) { int? }
  #     end
  #   end
  module Result
    class << self
      def [](*arguments)
        WithResult.new(*arguments)
      end
    end

    # @private
    class WithResult < Module
      def initialize(*args)
        @result_fields = args
      end

      def included(base)
        base.extend ClassMethods
        base.include Interface
        base.define_result_klass(*@result_fields)
      end

      # @private
      module Interface
        def result
          @result ||= self.class.result_klass.new
        end

        def call(**kwargs)
          before_call!(**kwargs)
          _call
          result
        end
      end

      # @private
      class ResultKlass
        class << self
          def define_fields(*fields)
            fields.each do |field|
              attr_accessor field
            end
          end
        end
      end

      # @private
      module ClassMethods
        attr_accessor :result_klass

        def define_result_klass(*fields)
          self.result_klass = Dry::Core::ClassBuilder.new(name: 'Result', parent: ResultKlass, namespace: self).call
          result_klass.define_fields(*fields)
        end
      end
    end
  end
end
